#pragma rtGlobals=1		// Use modern global access method.
#pragma version = 0.25
#pragma IgorVersion = 6.11
#initFunctionName "Init_herix()"
#pragma ModuleName=herix
#include "FWHM", version>=2.10
#include "spec", version>=2.44

strConstant DefaultPreferedHerixDetector = "Ana5"
strConstant photonCounters="detector;scint;NaI;oxfo;oxford;Bicr;Bicron;harmonic;Det_bot;Det_top;top_harmonic;Ana1;Ana2;Ana3;Ana4;Ana5;Ana6;Ana7;Ana8;Ana9;"
Static Constant maxAnalyzerNum = 9


Menu "Herix"
	"Find Phonon Energy...",FindPhononEnergy(0)
	SubMenu "Resolution Wave"
	MenuItemIfWaveClassExists("Change Resolution Wave...","ResolutionEnergyMeasured","DIMS:1"), herix#ReSetResolutionWave($"")
		"  Create Resolution Wave from spec scan...", herix#CreateNewResolutionWaveFromSpec(-1)
		"-"
		"  OLD: Set Resolution Wave...", herix#SetResolutionWave($"")
	End
	"  Set-Change the Prefered Herix Detector...",ChangePreferedHerixDetector("")
	"hkl of an Analyzer...", anlyzerN_hkl(NaN,NaN)
	SubMenu "Graph Many"
		"Make Layouts of Many Graphs...", ManyLayouts(NaN,"")
		"  Select a Range of Graph Numbers...", print herix#SelectRangeOfGraphNumbers(NaN,NaN)
	End
	"-"
End



// menu active only if top graph is a herix energy scan
Function/S MenuItemIfTopGraphHerixEscan(item)
	String item
	String gName = StringFromList(0,WinList("*",";","WIN:1,VISIBLE:1"))
	String traceName="", traceList=TraceNameList(gName,";",1)
	Variable i=0
	for (i=0;i<ItemsInList(traceList);i+=1)
		traceName = StringFromList(i,traceList)
		if (!StringMatch(traceName,"fit_"))
			return item
		endif
		traceName = ""
	endfor
	if (strlen(traceName)<1)					// no trace
		return "("+item
	endif

	Wave yw=TraceNameToWaveRef(gName,traceName)
	String cmd = StrVarOrDefault(GetWavesDataFolder(yw,2)+"specCommand","")
	if (strsearch(cmd,"herixE",0,2)>0)
		return "("+item
	endif
	return item
End



Proc Herix_Style() : GraphStyle
	HerixStyle("")
EndMacro
//
Function HerixStyle(gName)
	String gName
	ModifyGraph/W=$gName gfSize=18
	ModifyGraph/Z/W=$gName tick=2, minor=1, standoff=1
	ModifyGraph/W=$gName lowTrip=0.001
	if (strlen(AxisInfo(gName, "top"))<1)
		ModifyGraph/W=$gName/Z mirror(bottom)=1
	endif  
  	if (strlen(AxisInfo(gName, "right"))<1)
		ModifyGraph/W=$gName/Z mirror(left)=1
	endif

	// find the top wave on graph, and try to label the axes
	String wList = ImageNameList(gName,";")		// first assum an image
	if (strlen(wList))
		Wave w=ImageNameToWaveRef(gName,StringFromList(0,wList) )
	else												// next try a line plot
		wList = TraceNameList(gName,";",1)
		Wave w=TraceNameToWaveRef(gName,StringFromList(0,wList))
	endif
	if (WaveExists(w))
		String wnote=note(w)
		String left = StringByKey("GraphAxisLabelVert", wnote ,"=" )
		String bot = StringByKey("GraphAxisLabelHoriz", wnote ,"=" )
		String infoStr=ImageInfo(gName,NameOfWave(w),0)
		if (strlen(infoStr)<1)
			infoStr = TraceInfo(gName,NameOfWave(w),0)
		endif
		String leftName = StringByKey("YAXIS",infoStr)
		String botName = StringByKey("XAXIS",infoStr)
		Variable setHoriz = strlen(AxisLabelFromGraph(gName,w,botName))==0	// do not re-set if alreay labeled
		Variable setVert = strlen(AxisLabelFromGraph(gName,w,leftName))==0
		if (strlen(left) && setVert && strlen(leftName))
			Label $leftName left
		endif
		if (strlen(bot) && setHoriz && strlen(botName))
			Label $botName bot
		endif
	endif
End



Function FindPhononEnergy(scanNum,[printIt])
	Variable scanNum
	Variable printIt
	printIt = ParamIsDefault(printIt) ? 0 : printIt
	printIt = numtype(printIt) ? 0 : !(!printIt)
	Variable scanNumIn = scanNum

	if (scanNum<1 || numtype(scanNum))
		scanNum=NumVarOrDefault("root:Packages:spec:lastScan", 1)
		Prompt scanNum,"spec scan number"
		DoPrompt "give a scan number", scanNum
		if (V_flag)										// cancelled after prompt
			return NaN
		endif
	endif
	if (scanNum<1 || numtype(scanNum))
		Print "scan number must be >0"
		return NaN
	endif

	String folderName = Find_specDataFolder(scanNum)	// get the folder for the spec scan
	if (!cmpstr(folderName,"bad dir"))
		Print "the data folder does not exist"
		return NaN										// finally give up
	endif

	Wave xw = $(folderName+StrVarOrDefault(folderName+"xAxisName",""))
	Wave yw = $(folderName+StrVarOrDefault(folderName+"yAxisName",""))
	String list = FitPhononPeaksEnergy(xw,yw)

	scanNum=NumberByKey("SCAN_N",list,"=")
	Variable bad=NumberByKey("bad",list,"=")
	Variable p0Lo=NumberByKey("p0Lo",list,"="), p0Hi=NumberByKey("p0Hi",list,"=")
	Variable lo=NumberByKey("lo",list,"="), loErr=NumberByKey("loErr",list,"=")
	Variable loFWHM=NumberByKey("loFWHM",list,"="), loFWHMerr=NumberByKey("loFWHMerr",list,"=")
	Variable hi=NumberByKey("hi",list,"="), hiErr=NumberByKey("hiErr",list,"=")
	Variable hiFWHM=NumberByKey("hiFWHM",list,"="), hiFWHMerr=NumberByKey("hiFWHMerr",list,"=")
	Variable dE=NumberByKey("dE",list,"="), dEerr=NumberByKey("dEerr",list,"=")

	if (strlen(GetRTStackInfo(2))==0 || printIt)
		String units = WaveUnits(xw,1), range
		if (!(scanNum == scanNumIn))
			printf "   for spec scan #%g\r",scanNum
		endif
		sprintf range,"[%g, %g %s]",xw[0],xw[p0Lo],units
		if (numtype(lo))
			print "Lo Peak:  Fit Failed,  using range: ",range
		else
			printf "Lo Peak:  %s %s,  FWHM = %s,   range = %s\r",ValErrStr(lo,loErr,sp=1),units, ValErrStr(loFWHM,loFWHMerr,sp=1),range
		endif
		sprintf range,"[%g, %g %s]",xw[p0Hi],xw[numpnts(yw)-1],units
		if (numtype(hi))
			print "Hi Peak:  Fit Failed,  using range: ",range
		else
			printf "Hi Peak:  %s %s,  FWHM = %s,   range = %s\r",ValErrStr(hi,hiErr,sp=1),units, ValErrStr(hiFWHM,hiFWHMerr,sp=1),range
		endif
		if (bad)
			print "Failed to find Phonon Energy!"
		else
			printf "Phonon Energy = %s %s\r",ValErrStr(dE,dEerr,sp=1),units
		endif
	endif
	return dE
End
//
Static Function/T FitPhononPeaksEnergy(xw,yw)
	Wave xw,yw
	if (!WaveExists(xw) || !WaveExists(yw))
		return ""
	elseif (numpnts(xw) != numpnts(yw))
		return ""
	endif

	Variable testWidth = 5
	Variable p0=BinarySearch(xw,0)
	WaveStats/M=1/R=[p0-testWidth,p0+testWidth]/Q/Z yw
	p0 = V_maxloc
	if (p0<testWidth || numpnts(xw)<(2*testWidth) || V_npnts!=(2*testWidth+1))
		return ""
	endif

	Variable p0Lo = WalkToMin(p0,yw,-1)
	Variable p0Hi = WalkToMin(p0,yw,+1)
	if (p0Lo<testWidth || p0Hi>(numpnts(xw)-testWidth))
		return ""
	endif

	//	Wave ywErr = $(GetWavesDataFolder(yw,2)+"+err")
	Variable lo=NaN, loFWHM=NaN, loErr=NaN, loFWHMerr=NaN
	Variable hi=NaN, hiFWHM=NaN, hiErr=NaN, hiFWHMerr=NaN
	Variable V_FitOptions=2+4, V_FitError=0
	//	if (WaveExists(ywErr))
	//		CurveFit/Q lor yw[0,p0Lo] /X=xw/W=ywErr	/I=1
	//	else
			CurveFit/Q lor yw[0,p0Lo] /X=xw
	//	endif
	Wave W_coef=W_coef, W_sigma=W_sigma
	if (!V_FitError)
		lo = W_coef[2]
		loFWHM = 2*sqrt(W_coef[3])
		loErr=W_sigma[2]
		loFWHMerr = abs(sqrt(W_coef[3]+W_sigma[3]) - sqrt(W_coef[3]-W_sigma[3]))
	endif

	V_FitError = 0
	//	if (WaveExists(ywErr))
	//		CurveFit/Q lor yw[p0Hi,numpnts(xw)-1] /X=xw/W=ywErr/I=1
	//	else
			CurveFit/Q lor yw[p0Hi,numpnts(xw)-1] /X=xw
	//	endif
	if (!V_FitError)
		hi = W_coef[2]
		hiFWHM = 2*sqrt(W_coef[3])
		hiErr = W_sigma[2]
		hiFWHMerr = abs(sqrt(W_coef[3]+W_sigma[3]) - sqrt(W_coef[3]-W_sigma[3]))
	endif

	Variable dE = abs(hi-lo)/2
	Variable dEerr = sqrt(loErr^2 + hiErr^2)
	KillWaves/Z W_coef, W_sigma

	Variable bad = numtype(lo+loErr + hi+hiErr + loFWHM+loFWHMerr + hiFWHM+hiFWHMerr) > 0
	bad = bad || (lo<=WaveMin(xw)) || (lo>=xw[p0Lo])
	bad = bad || (hi>=WaveMax(xw)) || (hi<=xw[p0Hi])
	bad = bad || dE<dEerr
	bad = bad || (loFWHMerr > loFWHM) || (hiFWHMerr > hiFWHM)
	dE = bad ? NaN : dE

	String out=""
	Variable scanNum = NumberByKey("SCAN_N",note(xw),"=")
	if (scanNum>0)
		out = ReplaceNumberByKey("SCAN_N",out,scanNum,"=")
	endif
	out = ReplaceNumberByKey("bad",out,bad,"=")
	out = ReplaceNumberByKey("p0Lo",out,p0Lo,"=")
	out = ReplaceNumberByKey("p0Hi",out,p0Hi,"=")

	if (numtype(lo)==0)
		out = ReplaceNumberByKey("lo",out,lo,"=")
		out = ReplaceNumberByKey("loErr",out,loErr,"=")
		out = ReplaceNumberByKey("loFWHM",out,loFWHM,"=")
		out = ReplaceNumberByKey("loFWHMerr",out,loFWHMerr,"=")
	endif
	if (numtype(hi)==0)
		out = ReplaceNumberByKey("hi",out,hi,"=")
		out = ReplaceNumberByKey("hiErr",out,hiErr,"=")
		out = ReplaceNumberByKey("hiFWHM",out,hiFWHM,"=")
		out = ReplaceNumberByKey("hiFWHMerr",out,hiFWHMerr,"=")
	endif
	if (numtype(dE)==0)
		out = ReplaceNumberByKey("dE",out,dE,"=")
		out = ReplaceNumberByKey("dEerr",out,dEerr,"=")
	endif

	return out
End
//
Static Function WalkToMin(p0,yw,direction)
	Variable p0
	Wave yw
	Variable direction		// must be -1 or +1
	direction = sign(direction)

	Variable pEnd = (direction<0) ? 0 : DimSize(yw,0)-1		// check from p0 to pEnd
	Variable i, yMin=Inf, pMin=NaN
	for (i=p0; i!=pEnd; i+=direction)
		if (yw[i]<yMin)
			pMin = i
			yMin = yw[i]
		elseif (numtype(pMin)==0)
			break
		endif
	endfor
	return pMin
End



// find the center of the spectrum by folding it and calculating the correlation
// the center is at the max in the correlation
// returns point number of center (not x-value)
Function FindCenterOfSpectrum(yw,xw,hw)
	Wave yw, xw
	Variable hw
	if (!WaveExists(yw) || numtype(hw) || hw<=0)
		return NaN
	endif

	Variable imid
	if (WaveExists(xw))					// x-wave given
		imid = BinarySearch(xw,0)
	else										// y-wave is assumed scaled
		imid = -DimOffset(yw,0)/DimDelta(yw,0)
	endif

	hw = round(abs(hw))
	if (hw > imid-5)						// wave too show for this hw
		return NaN
	endif

	Make/N=(imid-hw)/D/FREE corrPlus, corrNeg
	Make/N=(2*hw+1)/D/FREE corr=0
	Variable ic, N=DimSize(yw,0), xc
	for (ic=-hw; ic<=hw; ic+=1)
		corrPlus = yw[imid+ic+p]
		corrNeg = yw[imid+ic-p]
		MatrixOp/FREE cc = sum(corrPlus * corrNeg)
		corr[ic+hw] = cc[0]
	endfor
	WaveStats/Q/M=1 corr
	return V_maxloc-hw+imid			// point number of center

//	ic = V_maxloc-hw+imid
//	if (WaveExists(xw))					// x-wave given, get the x-value
//		xc = xw[ic]
//	else										// y-wave is assumed scaled
//		xc = imid*DimDelta(yw,0) + DimOffset(yw,0)
//	endif
End



Function/WAVE anlyzerN_hkl(scanNum,analyzerN,[printIt,increment])
	Variable scanNum				// spec scan number
	Variable analyzerN			// analyzer number, probably in [1,maxAnalyzerNum]
	Variable printIt
	Variable increment			// increment the analyzer number and call again, used to show all analyzers
	printIt = ParamIsDefault(printIt) || numtype(printIt) ? strlen(GetRTStackInfo(2))==0 : printIt
	increment = ParamIsDefault(increment) || numtype(increment) ? 0 : increment

	if (analyzerN>maxAnalyzerNum)
		return $""
	elseif (numtype(scanNum+analyzerN) || scanNum<1 || analyzerN<1 || analyzerN>maxAnalyzerNum)
		scanNum = numtype(scanNum) || scanNum<1 ? NumVarOrDefault("root:Packages:spec:lastScan",5) : scanNum
		analyzerN = numtype(analyzerN) || analyzerN<1 ? 5 : analyzerN
		Prompt scanNum,"spec scan number"
		Prompt analyzerN,"analyzer number", popup,expandRange("1-"+num2istr(maxAnalyzerNum),"")+";all;"
		DoPrompt "Analyzer",scanNum,analyzerN 
		if (V_flag)
			return $""
		endif
		if (analyzerN > maxAnalyzerNum)
			increment = 1
			analyzerN = 1
		endif
		printIt = 1
	endif

	String pre="Analyzer"+num2istr(analyzerN)+"_"
	Make/N=3/D/FREE hkl = {specInfo(scanNum,pre+"H"),specInfo(scanNum,pre+"K"),specInfo(scanNum,pre+"L")}
	if (printIt && numtype(sum(hkl))==0)
		printf "hkl(#%d, ana%d) = %s\r",scanNum,analyzerN,vec2str(hkl,zeroThresh=1e-5)
	endif

	String wnote="waveClass=hkl;"
	wnote = ReplaceNumberByKey("SCAN_N",wnote,scanNum,"=")
	wnote = ReplaceNumberByKey("AnalyzerNum",wnote,analyzerN,"=")
	Note/K hkl, wnote
	if (numtype(sum(hkl)))
		return $""
	elseif (increment)				// used when "all" was selected
		analyzerN += 1
		anlyzerN_hkl(scanNum,analyzerN,printIt=printIt,increment=increment)
	endif
	return hkl
End



Function extraSpecReadProcess(extra)
	Variable extra
	if (!extra)
		return 0
	endif

	Variable ringPeriod = 3.68258537e-6 		// ring period, time between P0 pulses (s)

	SVAR specCommand=specCommand

	if (exists("ICO_C")==1)					// set Io wave if it exists
		Wave wI0 = ICO_C
	elseif (exists("I0")==1)
		Wave wI0 = I0
	elseif (exists("Io")==1)
		Wave wI0 = Io
	elseif (exists("I0_mon")==1)
		Wave wI0 = I0_mon
	endif

	if ((exists("sig")!=1) && WaveExists(wI0))	// make a normalized wave
		String fldr= GetDataFolder(0)
		Variable scanNum = str2num(fldr[4,inf])
		Variable filterTrans=specInfo(scanNum,"filterTrans")
		filterTrans = numtype(filterTrans) ? 1 : filterTrans
		if (exists("filter"))
			Wave filterWave=filter
			filterTrans = 1
		endif
		if (WaveDims(wI0)==1)
			SVAR axisName=yAxisName
			if (!SVAR_EXISTS(axisName))
				SVAR axisName=specYwave
			endif
		elseif (WaveDims(wI0)==2)
			SVAR axisName=zAxisName
		else
			DoAlert 0,"cannot yet deal with 3-d data"
			return 1
		endif
		axisName = StringFromList(0,axisName)	// only support one y wave here
		if (exists("corr"))
			Wave corr = corr
			WaveStats/Q corr
			axisName = SelectString(V_max!=0 || V_min!=0, axisName,"corr")
		endif
		String counterName = axisName

		Duplicate $counterName sig
		Wave sig_err=$"", harm=$""
		// list of names that are probably photon counting detectors (obviously I0 is not in this list)
		Variable isPhotons = FindListItem(counterName,photonCounters) >= 0
		WAVE seconds=seconds
		String dark_Io_Name = StrVarOrDefault("root:Packages:spec:dark_Io_Name","")
		Variable darkRate = specInfo(scanNum,dark_Io_Name)	// dark rate in I0
		darkRate = numtype(darkRate) ? 0 : darkRate

		if (isPhotons)								// photon counter
			Duplicate/O $counterName sig_err // changing sig, so MUST change sig_err
			Wave harm=harmonic
			if (WaveExists(harm))				// correct dead time with harmonic
				sig += 2*harm[p]
				sig_err = 1/sqrt(sig)			// here, sig_err is FRACTIONAL error in sig, assuming root(N) statistics
			elseif (exists("deadtime")==6 && exists("seconds")==1)
				sig_err = 1/sqrt(sig)			// here, sig_err is FRACTIONAL error in sig, assuming root(N) statistics
				FUNCREF deadtimeProto  dtFunc  = $"deadtime"	// if no harmonic, try deadtime function
				sig /= seconds
				sig = dtFunc(sig)					// dead time works on counts/second only
				sig *= seconds
			else
				sig_err = 1/sqrt(sig)			// here, sig_err is FRACTIONAL error in sig, assuming root(N) statistics
			endif
			sig_err = numtype(sig_err) ? 1 : sig_err	// in case sig is 0
		endif

		Variable scale = NumVarOrDefault("root:Packages:spec:scaleFactor",1)
		sig *= scale

		if (exists("seconds")==1)
			sig = sig / max(1,wI0-seconds*darkRate) / filterTrans
		else
			sig = sig / wI0 / filterTrans
		endif
		if (WaveExists(filterWave))
			sig /= filterWave
		endif
		sig_err *= sig									// sig value is now finished, change fracional error to actual error

		String noteStr = ReplaceStringByKey("waveClass",note(sig),"specWaveSig","=")
		noteStr = ReplaceStringByKey("GraphAxisLabelVert", noteStr, WordToGreek(axisName)+" / I\\Bo\\M", "=")
		noteStr = ReplaceStringByKey("sigSourceWave",noteStr,counterName,"=")
		noteStr = ReplaceStringByKey("I0Wave",noteStr,NameOfWave(wI0),"=")
		if (numtype(scale)==0 && scale != 1)
			noteStr = ReplaceNumberByKey("scaleFactor",noteStr,scale,"=")
		endif
		if (WaveExists(sig_Err))				// have an error wave, put path to it into sig wave note
			noteStr = ReplaceStringByKey("ErrorWave",noteStr,NameOfWave(sig_Err),"=")
		endif
		if (WaveExists(harm))					// using the harmonic wave, put path to it into sig wave note
			noteStr = ReplaceStringByKey("harmonicWave",noteStr,NameOfWave(harm),"=")
		endif
		Note/K sig, noteStr

		axisName = "sig"
		if (StringMatch(WaveUnits(sig,WaveDims(sig)),"count*"))
			String str = ""
			if (scale!=1)
				sprintf str,"x %.4g",scale
				str = ReplaceString("e+0",str,"e")
				str = ReplaceString("e-0",str,"e-")
			endif
			SetScale d 0,0,str, sig
		endif

		noteStr = ReplaceStringByKey("waveClass",note(sig),"specWaveSigError,Error","=")
		noteStr = RemoveByKey("ErrorWave",noteStr,"=")
		noteStr = ReplaceStringByKey("ErrorWaveSource",noteStr,NameOfWave(sig),"=")
		Note/K sig_err, noteStr
	endif
End
//
Function deadtimeProto(rate)
	Variable rate
	return rate
End



Function extraSpecRead(fileVar)
	Variable fileVar							// file ref number

	String line1,line2						// line of input from file
	Variable i

	FStatus fileVar							// this section special for reading #BL and #B lines
	Variable FilePos = V_filePos			// this should be the start of the scan

	line1 = FindDataLineType(fileVar,"#G4 ",1)
	if (strlen(line1)>2)
		line1 = line1[4,inf]
		SetValuesIntoList("--;--;--;wavelength",line1,"specValues","  ")
	endif

	FSetPos fileVar, FilePos					// reset file position to start of scan
End




Function addMoreAnnotation2specPlot(scanNum)
	Variable scanNum
	if (!(scanNum>0))
		return 1
	endif

	Variable T_sample=specTemperatureHerix(scanNum)
	Variable H=specInfo(scanNum,"H"), K=specInfo(scanNum,"K"), L=specInfo(scanNum,"L")
	String str="", add=""
	if (numtype(H+K+L)==0)
		add += SelectString(strlen(add),"","\r")
		sprintf str, "\\Zr075hkl = (%.3f, %.3f, %.3f)\\M",H,K,L
		add += str
		Make/N=3/FREE qvec={H,K,L}
		qvec = abs(qvec - round(qvec))
		qvec = abs(qvec)<1e-4 ? 0 : qvec
		sprintf str, "\\Zr075\\f02q\\f00 = (%s)\\M",vec2str(qvec,places=3,bare=1)
		add += "\r"+str
	endif
	if (numtype(T_sample)==0 && T_sample>0)
		add += SelectString(strlen(add),"","\r")
		sprintf str, "T=%.2f K",T_sample
		add += str
	endif

	String folderName = Find_specDataFolder(scanNum)	// get the folder for the spec scan
	Wave xw = $(folderName+StrVarOrDefault(folderName+"xAxisName",""))
	Wave yw = $(folderName+StrVarOrDefault(folderName+"yAxisName",""))
	String list = FitPhononPeaksEnergy(xw,yw)
	Variable dE=NumberByKey("dE",list,"="), dEerr=NumberByKey("dEerr",list,"=")

	if (numtype(dE+dEerr)==0 && dE>0)
		add += SelectString(strlen(add),"","\r")
		sprintf str, "\\Zr075E = %s\\M",ValErrStr(dE,dEerr,sp=1)
		add += str
	endif
	if (strlen(add))
		AppendText/N=text0 add
	endif

// This functionality is now in spec.ipf, see  ModifyTopTraceOnSpecPlotProto()
//	String traceList=TraceNameList("",";",1)		// traces on top graph
//	if (ItemsInList(tracelist)==1)					// only one trace on graph
//		ModifyGraph rgb[0]=(0,0,65535)			// blue
//		if (DimSize(yw,0)<150)					// use plain line for more than 150 points
//			ModifyGraph mode[0]=4,marker[0]=19,lstyle[0]=1	// otherwise dots + dotted line
//		endif
//	endif
End


Function specTemperatureHerix(scanNum)								// return Temperature for a spec scan
	Variable scanNum
	Variable T_sample = TemperatureInCurrentFolder(Find_specDataFolder(scanNum))
	return T_sample
End
//
Static Function TemperatureInCurrentFolder(folderName)	// get spec Temperature in the given folder
	String folderName
	if (strlen(folderName))				// ensure non-empty folderName ends with ":"
		Variable i = char2num(folderName[strlen(folderName)-1])
		if (i != 58)
			folderName += ":"
		endif
	endif

	String Tname = StrVarOrDefault("root:Packages:DampedHarmonic:Tname",Temperature_PVorWave)
	Variable T_sample=NaN
	Wave Twave = $(folderName+Tname)
	if (WaveExists(Twave) && numpnts(Twave)>0)						// first try wave named Tname
		T_sample = StatsMedian(Twave)
	endif
	if (!(T_sample>0))														// not in wave, try EPICS_PVs
		String list=""
		list += StrVarOrDefault(folderName+"EPICS_PVs","")
		list += StrVarOrDefault(folderName+"specValues","")
		T_sample = NumberByKey(Tname,list)								// second try PV named Tname
		if (!(T_sample>0))
			T_sample = NumberByKey("T_sample",list)					// third and last try for "T_sample"
		endif
	endif
	T_sample = T_sample > 0 ? T_sample : NaN						// no default, and bad values are allways NaN
	return T_sample
End
//
//	Function specTemperatureHerix(scanNum)
//		Variable scanNum
//
//		Variable T_sample=specInfo(scanNum,"T_Sample")
//		if (!(T_sample>0))
//			Wave Twave = $(Find_specDataFolder(scanNum)+"T_sample_LS340")
//			if (WaveExists(Twave) && numpnts(Twave)>0)
//				WaveStats/M=1/Q Twave
//				T_sample = V_avg
//			endif
//		endif
//		T_sample = T_sample > 0 ? T_sample : NaN						// no default, and bad values are all NaN
//		return T_sample
//	End




Function extraSpecUserReadProcess()
	SVAR specCommand=specCommand
	if (strsearch(specCommand,"ascan  herixE ",0)==0)
		if (exists("zAxisName")==2)
			SVAR axisName = zAxisName
		else
			SVAR axisName = yAxisName
		endif
		if (WhichListItem(axisName,"Seconds;PIN_B")>=0)
			String detName = StrVarOrDefault("root:Packages:spec:PreferedHerixDetector",DefaultPreferedHerixDetector)
			if (exists(detName)==1)	// the wave exists, so I can use it
				axisName = detName
			endif
		endif
	endif

	String wList=WaveList("Ana*",";","")
	Variable i, N=ItemsInList(wList)
	for (i=1;i<N;i+=1)
		Wave ana= $StringFromList(i,wList)
		SetScale d 0,0,"counts", ana
	endfor
	Wave ene=HerixE
	if (WaveExists(ene))
		SetScale d 0,0,"meV", ene
	endif
End



Function/T ChangePreferedHerixDetector(detName)		// Change Prefered Herix Detector for Plotting
	String detName
	String detName0 = StrVarOrDefault("root:Packages:spec:PreferedHerixDetector","")	// current default

	if (strlen(detName)==0)
		detName = SelectString(strlen(detName0),DefaultPreferedHerixDetector,detName0)
		Prompt detName,"Name of Herix Detector to Plot"
		DoPrompt "Prefered Detector", detName
		if (V_flag)
			return ""
		endif
	endif
	if (StringMatch(detName,detName0))
		printf "Leaving Prefered Herix Detector UNchanged as \"%s\"\r",detName
	else
		printf "Changing Prefered Herix Detector from  \"%s\" --> \"%s\"\r",detName0,detName
		String /G root:Packages:spec:PreferedHerixDetector = detName
	endif
	return detName
End






//  ================================================================================  //
//  ========================== Start Make Resolution Wave ==========================  //

Static Function/WAVE ReSetResolutionWave(rNew)
	Wave rNew
	if (WaveExists(rNew))
		Duplicate/O rNew, root:resolutionWave
		return rNew
	endif

	Wave rOld=root:resolutionWave
	String oldNote="", old="No Old resolution Set"
	if (WaveExists(rOld))
		oldNote = note(rOld)
		old = StringByKey("source",oldNote,"=")
	endif

	String list=WaveListClass("ResolutionEnergyMeasured","*","DIMS:1"), resName=""
	list = RemoveFromList("resolutionWave",list)
	if (ItemsInList(list) < 1)
		DoAlert 0, "No resolution waves found to choose from"
		return $""
	elseif (ItemsInList(list)==1)
		resName = StringFromList(0,list)
	else
		resName = StringByKey("copiedFrom",oldNote,"=")
		Prompt resName, "New Resolution Wave", popup, list
		DoPrompt "Choose Resolution", resName
		if (V_flag)
			return $""
		endif
	endif
	Wave rNew = $resName
	if (!WaveExists(rNew))
		DoAlert 0, "The wave \""+resName+"\" was not found"
		return $""
	endif

	String new, wnote=note(rNew)
	new = StringByKey("source",wnote,"=")
	Duplicate/O rNew, root:resolutionWave
	wnote = ReplaceStringByKey("copiedFrom",wnote,NameOfWave(rNew),"=")
	Note/K root:resolutionWave, wnote

	printf "\r Changed Resolution Wave from \"%s\" to \"%s\"\r\r",old,new
	return rNew
End


Static Function/WAVE CreateNewResolutionWaveFromSpec(scanNum)	// This makes the scaled & normalized resolution wave from a spec scan
	// make a normalized resolution wave, sum(resolutionWave) = 1
	Variable scanNum				// if <1 then you get to pick

	if (scanNum<1 || numtype(scanNum))			// ask for scan number
		scanNum=NumVarOrDefault("root:Packages:spec:lastScan", 1)
		Prompt scanNum,"spec scan number"
		DoPrompt "give a scan number", scanNum
		if (V_flag)											// cancelled after prompt
			return $""
		endif
	endif
	if (scanNum<1 || numtype(scanNum))
		Print "scan number must be >0"
		return $""
	endif

	Variable xlo=-30, xhi=30, NresWave=400
	Make/N=(NresWave)/D/FREE resWave=0
	SetScale/I x , xlo, xhi,"meV", resWave
	String wnote="waveClass=ResolutionEnergyMeasured;", source, yname, xname, rname
	sprintf source,"%s  scan #%d", specTime2Igor(specInfoT(scanNum,"timeWritten"),-2),scanNum
	sprintf yname "root:spec%d:sig", scanNum
	sprintf xname "root:spec%d:HerixE", scanNum
	Wave yres=$yname, xres=$xname
	Wave Wc = CalcSimplePeakParameters(yres,xres,0)
	Duplicate/FREE xres xresOff
	xresOff -= Wc[2]

	Variable N=DimSize(xresOff,0), ilo, ihi, su
	ilo = limit(x2pnt(resWave,xresOff[0]), 0, NresWave-1)
	ihi = limit(x2pnt(resWave,xresOff[N-1]), 0, NresWave-1)
	resWave[ilo,ihi] = interp(x,xresOff,yres)
	WaveStats/R=[ilo,ihi]/M=1/Q resWave
	resWave[ilo,ihi] -= V_min						// set smallest value to zero
	Wave Wc = CalcSimplePeakParameters(resWave,$"",0)
	Variable FWHM=Wc[3], Eoffset=Wc[2]			// Wc = {bkg, amplitude, x0, FWHM, net, COM}   Simple peak stats
	su = sum(resWave)
	resWave /= su

	wnote = ReplaceNumberByKey("FWHM",wnote,FWHM,"=")
	wnote = ReplaceNumberByKey("Eoffset",wnote,Eoffset,"=")
	wnote = ReplaceStringByKey("source",wnote,source,"=")
	Note/K resWave, wnote

	String timeWritten = specInfoT(12,"timeWritten")
	sprintf rname, "resolution_%s_%s_scan%d", StringFromList(0,timeWritten," "), StringFromList(1,timeWritten," "), scanNum
	rname = CleanupName(rname,0)
	Duplicate/O resWave, $rname
	printf "Making resolution \"%s\"  using '%s', with FWHM = %g meV,  Eoffset = %g meV\r",rname,source,FWHM,Eoffset
	return $rname
End


Static Function/WAVE SetResolutionWave(ww)	// This makes the scaled & normalized resolution wave
	// make a normalized resolution wave, sum(resolutionWave) = 1
	Wave ww					// an optional wave, uses wave's MODTIME, to pick the resolution wave

	if (Exists("root:resolutionWave")==1)
		DoAlert 1, "resolutionWave exists, make a new one?"
		if (V_flag != 1)
			return root:resolutionWave
		endif
	endif

	Variable epoch=NumberByKey("MODTIME",WaveInfo(root:spec122:HerixE, 0))
	Wave yres0=root:Normalized_Resolution5, xres0=root:HerixE_Resolution5

	Variable pick = epoch<date2secs(2014,4,1) ? 2 : 1
	String pickMenu="Oct 2014;Oct 2013"
	if (WaveExists(yres0) && WaveExists(xres0))		// can add the pick=3 & make it default
		pickMenu += ";use existing Normalized_Resolution5 wave"
		pick = 3
	endif

	Prompt pick "Which resolution to use", popup, pickMenu
	DoPrompt "Which Resolution?", pick
	if (V_flag)
		return $""
	endif

	Make/N=400/D/O root:resolutionWave/Wave=resolutionWave
	SetScale/I x , -30, 30,"meV", resolutionWave
	String wnote="waveClass=ResolutionEnergyMeasured;"
	String source=""

	if (pick==1)			// Oct 2014 resolution
		source = "Oct 2014 from Ayman"
		resolutionWave[0]= {0,0.000140468,0.0001382,0.000131074,0.000123476,0.000117941,0.0001122,0.00010509,9.66001e-05,8.99803e-05,8.75742e-05,8.64985e-05,8.6541e-05,9.03954e-05,9.81575e-05,0.000108375}
		resolutionWave[16]= {0.000118366,0.000126564,0.000132189,0.000132835,0.000129929,0.000126632,0.000122529,0.000117918,0.000114579,0.000112078,0.000107522,0.000101618,9.88137e-05,0.000102436,0.000110431}
		resolutionWave[31]= {0.000117413,0.000124281,0.000133988,0.000142315,0.000144889,0.000146105,0.000149797,0.000154865,0.000157717,0.000155844,0.000150809,0.000145906,0.000142742,0.000141797,0.000142698}
		resolutionWave[46]= {0.000143561,0.000146354,0.000153539,0.000162796,0.000171506,0.000179706,0.000188342,0.000196092,0.000200177,0.000202893,0.000208125,0.000212791,0.00021398,0.000212829,0.000210929}
		resolutionWave[61]= {0.00020512,0.000195386,0.000186656,0.000181925,0.000180625,0.00018039,0.000182776,0.00018964,0.000198025,0.000205647,0.000211579,0.000214121,0.00021326,0.000211773,0.000209923}
		resolutionWave[76]= {0.000203627,0.000193953,0.000184536,0.000179516,0.000177253,0.000170378,0.000164736,0.000167447,0.000177315,0.00019128,0.000204837,0.000217335,0.000229177,0.000234497,0.000233286}
		resolutionWave[91]= {0.000233174,0.000236432,0.000240347,0.000244062,0.000245215,0.000241195,0.000237132,0.000238724,0.000246279,0.000254933,0.000260745,0.000266399,0.000272915,0.000278255,0.000282822}
		resolutionWave[106]= {0.000289848,0.000297805,0.000300406,0.000298304,0.00029803,0.000300061,0.000300894,0.000301179,0.000306704,0.000317037,0.000322923,0.000326089,0.000332531,0.000340776,0.000350164}
		resolutionWave[121]= {0.000361101,0.000367466,0.000363993,0.000359906,0.000363467,0.000372257,0.000375983,0.000370698,0.000359875,0.000345326,0.000333883,0.000335516,0.000351486,0.000371769,0.000386161}
		resolutionWave[136]= {0.00039292,0.000401603,0.000416867,0.000434213,0.00045257,0.000472891,0.0004915,0.000509609,0.00053546,0.000565793,0.00059281,0.000617869,0.000643489,0.000660234,0.000670531}
		resolutionWave[151]= {0.000687813,0.000717708,0.000757405,0.000794954,0.000825818,0.000850383,0.00087133,0.000892353,0.000909159,0.000923048,0.000940409,0.000964587,0.000993459,0.00101194,0.000965973}
		resolutionWave[166]= {0.000982842,0.000999702,0.00101398,0.00105831,0.0011503,0.00124356,0.00133641,0.00142489,0.00150376,0.00156357,0.00165879,0.00177136,0.00191453,0.00204225,0.00215357,0.00238004}
		resolutionWave[182]= {0.00262669,0.00292656,0.00327923,0.00370085,0.00428814,0.00499922,0.00605581,0.0074914,0.00953995,0.0132295,0.0178692,0.0250658,0.0333842,0.0426877,0.0510512,0.0588333,0.0647174}
		resolutionWave[199]= {0.0676198,0.0673021,0.0632635,0.058026,0.0509221,0.0428139,0.0333692,0.0267274,0.0209287,0.0159354,0.0122817,0.00960015,0.00771859,0.00618074,0.00518998,0.00447161,0.00399698}
		resolutionWave[216]= {0.00350066,0.00301976,0.00271057,0.00246854,0.00230044,0.00213955,0.00198379,0.00184033,0.00172942,0.00163792,0.00155489,0.00146868,0.00136049,0.00129116,0.00125057,0.00115465}
		resolutionWave[232]= {0.00102048,0.000981884,0.000957816,0.000931403,0.000892755,0.000856788,0.00083252,0.00081463,0.000802817,0.000797804,0.0007916,0.000780901,0.000766609,0.00074694,0.000719295}
		resolutionWave[247]= {0.000684935,0.000653645,0.000627262,0.000597618,0.000570489,0.000565709,0.000577382,0.000582593,0.000572797,0.000555901,0.000543184,0.000532808,0.000523213,0.000517009,0.000509713}
		resolutionWave[262]= {0.000498631,0.000486016,0.000473017,0.000459118,0.000444996,0.000434084,0.000420769,0.000395027,0.000368457,0.000355999,0.000361292,0.000372448,0.000374822,0.000371549,0.000368012}
		resolutionWave[277]= {0.000359459,0.000347494,0.000337056,0.000325742,0.00031434,0.000308681,0.000306602,0.000301688,0.000295892,0.00029224,0.000283592,0.000265471,0.000243608,0.000230662,0.00022973}
		resolutionWave[292]= {0.000234419,0.000237416,0.000235291,0.000234411,0.000236944,0.00023602,0.000231406,0.000225568,0.000220214,0.000215788,0.000205037,0.000190576,0.000184116,0.000188316,0.000195054}
		resolutionWave[307]= {0.000197573,0.000197827,0.000195384,0.000192128,0.000191548,0.000191698,0.000190181,0.000186638,0.000186372,0.000189372,0.000186408,0.000178614,0.000175863,0.000181131,0.000188458}
		resolutionWave[322]= {0.00019004,0.000192502,0.000199924,0.00020366,0.000201077,0.000191253,0.000175704,0.00016398,0.000161164,0.00016184,0.000160957,0.000158688,0.00015676,0.000156409,0.000154464}
		resolutionWave[337]= {0.000147675,0.000143317,0.000146612,0.000151813,0.000153709,0.00015044,0.000145033,0.000140832,0.000135612,0.000126954,0.000115102,0.000108098,0.000110178,0.000112223,0.000111322}
		resolutionWave[352]= {0.000111737,0.000113288,0.000113742,0.000110511,0.000103417,9.51397e-05,8.89976e-05,8.67107e-05,8.85766e-05,9.36134e-05,0.000100233,0.000107777,0.000114728,0.000118604,0.000120053}
		resolutionWave[367]= {0.000121188,0.000122654,0.000123453,0.000123866,0.000124998,0.000123441,0.000117298,0.000108842,9.94304e-05,9.05711e-05,8.40359e-05,8.06701e-05,8.06679e-05,8.10125e-05,7.96922e-05}
		resolutionWave[382]= {7.83901e-05,7.72579e-05,7.55864e-05,7.43619e-05,7.45952e-05,7.6907e-05,8.12952e-05,8.79081e-05,9.42177e-05,9.55167e-05,9.09015e-05,8.63123e-05,8.5337e-05,8.68404e-05,8.96725e-05}
		resolutionWave[397]= {9.23308e-05,9.53686e-05,0}

	elseif (pick==2)			// Oct 2013 resolution
		source = "Oct 2013 from Ayman"
		Make/N=241/D/FREE yres
		yres[0]= {0.00232359,0.00261175,0.00225269,0.0020063,0.00204314,0.000353422,0.00183676,0.00148194,0.00184613,0.00138664,0.00163227,0.00311563,0.00216847,0.0030499,0.00229331,0.00102994}
		yres[16]= {0.00103848,0.00167618,0.00160053,0.00259333,0.00259537,0.0023355,0.00360795,0.00270858,0.00170235,0.00225607,0.00164427,0.00433384,0.00265271,0.00130053,0.00289437,0.00328019}
		yres[32]= {0.00374533,0.00321965,0.00402717,0.00393381,0.00402433,0.00288351,0.00265145,0.00263227,0.00387071,0.00364842,0.00141115,0.00448365,0.00463952,0.00393324,0.00383937,0.00235466}
		yres[48]= {0.00222308,0.00305656,0.00327743,0.00178123,0.00384351,0.00555378,0.00355234,0.00503061,0.00432572,0.00293939,0.00357338,0.00327616,0.00607471,0.00420812,0.00479003,0.00481779}
		yres[64]= {0.00554518,0.0045625,0.0046961,0.00465247,0.00586904,0.0071712,0.00357081,0.00569376,0.00501291,0.00720128,0.00720519,0.00421536,0.00787475,0.00697435,0.00592534,0.00341731}
		yres[80]= {0.00750675,0.00642255,0.00520941,0.00648819,0.0108921,0.0102212,0.0067778,0.00818764,0.00972624,0.0111019,0.01323,0.0129372,0.0131248,0.0134303,0.0147379,0.0114005,0.0177995}
		yres[97]= {0.0204886,0.0167144,0.0155784,0.0177203,0.0169769,0.0204038,0.0210214,0.0174468,0.0267516,0.0306759,0.0295941,0.0336471,0.0420168,0.0484627,0.058541,0.0650181,0.086798,0.134259}
		yres[115]= {0.187312,0.316721,0.499464,0.79745,0.988883,1,0.917746,0.656118,0.373256,0.248781,0.145384,0.104674,0.0629004,0.0656706,0.0522699,0.0381385,0.037,0.031272,0.0273148,0.0279348}
		yres[135]= {0.0238515,0.0233861,0.022022,0.0200947,0.0166599,0.0163559,0.0156019,0.0127385,0.0166074,0.0125535,0.016973,0.011837,0.00844706,0.0140022,0.011925,0.012574,0.00894305,0.00757134}
		yres[153]= {0.00795377,0.010063,0.00840022,0.0113259,0.0120054,0.00614049,0.00593489,0.00554258,0.00770408,0.00635867,0.00864287,0.00752536,0.00547843,0.00438766,0.00260499,0.00841665}
		yres[169]= {0.00777742,0.0047875,0.00545208,0.00330015,0.0042244,0.00409753,0.00385068,0.00365364,0.00584545,0.00275507,0.00462902,0.00364361,0.00322238,0.00303716,0.00425353,0.002716}
		yres[185]= {0.00262778,0.00353291,0.00191684,0.00646317,0.00262511,0.00262397,0.00130668,0.00437803,0.00291561,0.00260978,0.0038671,0.00237793,0.00596686,0.00320947,0.000994945,0.00164996}
		yres[201]= {0.00195832,0.00195909,0.00551081,0.00262169,0.00273406,0.00225905,0.00126822,0.00127717,0.00228875,0.00257003,0.00232761,0.00230617,0.00100713,0.00130975,0.0022499,0.0012913}
		yres[217]= {0.00191576,0.00132248,0.0014042,0.00238887,0.00269849,0.00254179,0.00193694,0.00164967,0.00171996,0.00163317,0.00130729,0.00196002,0,0.00167129,0.00159963,0.00159697,0.00159658}
		yres[234]= {0.000642121,0.00173492,0.00133285,0.00198561,0.00222618,0.00161349,0.000333778}

		Make/N=241/D/FREE xres
		xres[0]= {-29.9911,-29.748,-29.5049,-29.2618,-28.9883,-28.7453,-28.5022,-28.2591,-27.9856,-27.7425,-27.4994,-27.2563,-27.0132,-26.7398,-26.4967,-26.2536,-26.0105,-25.737,-25.4939,-25.2508}
		xres[20]= {-25.0078,-24.7647,-24.4912,-24.2481,-24.005,-23.7619,-23.4885,-23.2454,-23.0023,-22.7592,-22.4857,-22.2426,-21.9995,-21.7564,-21.5134,-21.2399,-20.9968,-20.7537,-20.5106,-20.2371}
		xres[40]= {-19.9941,-19.751,-19.5079,-19.2648,-18.9913,-18.7482,-18.5051,-18.262,-17.9886,-17.7455,-17.5024,-17.2593,-16.9858,-16.7427,-16.4997,-16.2566,-16.0135,-15.74,-15.4969,-15.2538,-15.0107}
		xres[61]= {-14.7373,-14.4942,-14.2511,-14.008,-13.7649,-13.4914,-13.2483,-13.0053,-12.7622,-12.4887,-12.2456,-12.0025,-11.7594,-11.4859,-11.2429,-10.9998,-10.7567,-10.5136,-10.2401,-9.99703}
		xres[81]= {-9.75394,-9.51085,-9.23737,-8.99429,-8.7512,-8.50811,-8.26502,-7.99154,-7.74846,-7.50537,-7.26228,-6.9888,-6.74571,-6.50263,-6.25954,-5.98606,-5.74297,-5.49988,-5.2568,-5.01371}
		xres[101]= {-4.74023,-4.49714,-4.25405,-4.01097,-3.73749,-3.4944,-3.25131,-3.00822,-2.76514,-2.49166,-2.24857,-2.00548,-1.76239,-1.48892,-1.24583,-1.00274,-0.759652,-0.486178,-0.243089,0,0.243089}
		xres[122]= {0.486178,0.759652,1.00274,1.24583,1.48892,1.76239,2.00548,2.24857,2.49166,2.76514,3.00822,3.25131,3.4944,3.73749,4.01097,4.25405,4.49714,4.74023,5.01371,5.2568,5.49988,5.74297}
		xres[144]= {5.98606,6.25954,6.50263,6.74571,6.9888,7.26228,7.50537,7.74846,7.99154,8.26502,8.50811,8.7512,8.99429,9.23737,9.51085,9.75394,9.99703,10.2401,10.5136,10.7567,10.9998,11.2429,11.4859}
		xres[167]= {11.7594,12.0025,12.2456,12.4887,12.7622,13.0053,13.2483,13.4914,13.7649,14.008,14.2511,14.4942,14.7373,15.0107,15.2538,15.4969,15.74,16.0135,16.2566,16.4997,16.7427,16.9858,17.2593}
		xres[190]= {17.5024,17.7455,17.9886,18.262,18.5051,18.7482,18.9913,19.2648,19.5079,19.751,19.9941,20.2371,20.5106,20.7537,20.9968,21.2399,21.5134,21.7564,21.9995,22.2426,22.4857,22.7592,23.0023}
		xres[213]= {23.2454,23.4885,23.7619,24.005,24.2481,24.4912,24.7647,25.0078,25.2508,25.4939,25.737,26.0105,26.2536,26.4967,26.7398,27.0132,27.2563,27.4994,27.7425,27.9856,28.2591,28.5022,28.7453}
		xres[236]= {28.9883,29.2618,29.5049,29.748,29.9911}
		resolutionWave = interp(x,xres,yres)

	elseif (pick==3)			// Oct 2013 resolution
		Wave yres0=root:Normalized_Resolution5, xres0=root:HerixE_Resolution5
		if (!WaveExists(yres0) || !WaveExists(xres0))
			print "ERROR -- Cannot find waves root:Normalized_Resolution5 and root:HerixE_Resolution5"
			return $""
		endif
		source = "Normalized_Resolution5 & HerixE_Resolution5"
		resolutionWave = interp(x,xres,yres)
	else
		return $""										// this should be impossible to hit
	endif

	Wave Wc = CalcSimplePeakParameters(resolutionWave,$"",0)
	Variable FWHM=Wc[3], Eoffset=Wc[2]			// Wc = {bkg, amplitude, x0, FWHM, net, COM}   Simple peak stats

	printf "Making resolution using '%s', with FWHM = %g meV,  Eoffset = %g meV\r",source,FWHM,Eoffset
	Variable su=sum(resolutionWave)
	resolutionWave /= su
	wnote = ReplaceNumberByKey("FWHM",wnote,FWHM,"=")
	wnote = ReplaceNumberByKey("Eoffset",wnote,Eoffset,"=")
	wnote = ReplaceStringByKey("source",wnote,source,"=")
	Note/K resolutionWave, wnote

	return resolutionWave
End

//  =========================== End Make Resolution Wave ===========================  //
//  ================================================================================  //



//  ================================================================================  //
//  ======================= Start Put Many Graphs on Layouts =======================  //

Function ManyLayouts(Nlayout,range,[printIt])
	Variable Nlayout
	String range
	Variable printIt
	printIt = ParamIsDefault(printIt) || numtype(printIt) ? strlen(GetRTStackInfo(2))==0 : printIt

	String NlayoutAllowed = "1;2;3;4;6;8;9;12;16;20"		// list of allowed values for Nlayout
	if (WhichListItem(num2str(Nlayout),NlayoutAllowed)<0)
		Nlayout = 4
		Prompt Nlayout, "Number of graphs on one Layout", popup, NlayoutAllowed
		DoPrompt "Layout", Nlayout
		if (V_flag)
			return 1
		endif
		Nlayout = str2num(StringFromList(Nlayout-1,NlayoutAllowed))
		printIt = 1
	endif

	if (ItemsInRange(range)<1)
		printIt = 1
		range = SelectRangeOfGraphNumbers(NaN,NaN)
	endif
	if (printIt)
		printf "ManyLayouts(%g,\"%s\")\t\t// Showing %g graphs\r", Nlayout, range, ItemsInRange(range)
	endif
	if (ItemsInRange(range)>1000)
		print "This is too many, stopping"
		DoAlert 0, "This is too many, stopping"
		return 1
	elseif (ItemsInRange(range)<1)
		print "range is empty, stopping"
		return 1
	elseif (WhichListItem(num2str(Nlayout),NlayoutAllowed)<0)
		printf "Nlayout cannot be %g, only one of \"%s\"\r",Nlayout,NlayoutAllowed
		return 1
	endif

	String gName, gList
	Variable i, m=NextInRange(range,-Inf)
	do
		gList=""
		for (i=0; i<Nlayout && numtype(m)==0; i+=1)
			sprintf gName, "Graph_%d", m
			// print i,"   ",gName
			if (ItemsInList(WinList(gName,";","WIN:1")))
				DoWindow/HIDE=? $gName			// is graph hidden?
				if (V_flag==2)
					DoWindow/HIDE=0 $gName		// make Visible graph if hidden
				endif
				DoWindow/F $gName					// graph already displayed, bring to front
			else
				Execute gName+"()"				// display the graph
			endif
			gList += gName + ";"
			DoUpdate
			AppendGraph2LayoutN(Nlayout,"landscape",gName)
			m = NextInRange(range,m)
		endfor
	while (numtype(m)==0)
	//	KillWindow winName
End
//
Static Function/S SelectRangeOfGraphNumbers(graphMin,graphMax)
	Variable graphMin,graphMax		// range of graph numbers to consider
	if (numtype(graphMin) || numtype(graphMax)==2 || graphMin<0 || graphMax<graphMin)
		graphMin = numtype(graphMin) || graphMin<0 ? 0 : graphMin
		graphMax = numtype(graphMax) || graphMax>graphMin ? Inf : graphMax
		Prompt graphMin, "Lowest Graph Number (>=0)"
		Prompt graphMax, "Hightest Graph Number (Inf gets all)"
		DoPrompt "Graph Range",graphMin, graphMax
		if (V_flag)
			return ""
		endif
	endif
	if (numtype(graphMin) || numtype(graphMax)==2 || graphMin<0 || graphMax<graphMin)
		return ""
	endif

	String mlist = ReplaceString("Graph_", MacroList("Graph_*",";","KIND:4,SUBTYPE:Graph"),"")
	String list=""
	Variable i,m
	for (i=0;i<ItemsInList(mlist);i+=1)
		m = str2num(StringFromList(i,mlist))
		if (m>=0 && numtype(m)==0 && mod(m,1)==0 && m>=graphMin && m<=graphMax)
			list += num2istr(m)+";"
		endif
	endfor
	list = SortList(list,";",2)
	String range = compressRange(list,";")
	return range
End

//  ======================== End Put Many Graphs on Layouts ========================  //
//  ================================================================================  //



Function Init_herix()
	specInitPackage()
	Variable /G root:Packages:spec:extraProcess=1		// extraSpecReadProcess(extra) exists here, so use it
	if (exists("root:Packages:spec:dark_Io_Name")!=2)
		// name of value from specInfo with the dark current in Io
		String /G root:Packages:spec:dark_Io_Name = ""
	endif
	if (exists("root:Packages:spec:PreferedHerixDetector")!=2)
		// name of wave with prefered Herix signal
		String /G root:Packages:spec:PreferedHerixDetector = DefaultPreferedHerixDetector
	endif
	return 0
End

// An Example, puts peak statistics on the plot
//
//Function addMoreAnnotation2specPlot(scanNum)
//	Variable scanNum
//
//	String fldr = Find_specDataFolder(scanNum)
//	Wave xw = $(fldr+StrVarOrDefault(fldr+"xAxisName",""))
//	Wave yw = $(fldr+StrVarOrDefault(fldr+"yAxisName",""))
//	if (!WaveExists(yw))
//		return 1
//	endif
//
//	String list = peakStats(yw,xw)
//	String str, units = SelectString(WaveExists(xw), WaveUnits(yw,0), WaveUnits(xw,-1))
//	if (strlen(list))
//		sprintf str,"\\Zr080FWHM = %.4g\rcenter = %.4g\rmax = %.4g\r  at  %g%s",NumberByKey("fwhm",list),NumberByKey("center",list),NumberByKey("max",list),NumberByKey("Xmax",list),units
//		TextBox/C/N=textPeak/F=0/B=1 str
//	endif
//
//	Wave Hw=$(fldr+"H"), Kw=$(fldr+"K"), Lw=$(fldr+"L")
//	if (!WaveExists(Hw) || !WaveExists(Kw) || !WaveExists(Lw))
//		return 1
//	endif
//	Variable m = NumberByKey("centerPnt",list)
//	sprintf str,"\\Zr080hkl = (%.3f, %.3f, %.3f)\\M",Hw[m],Kw[m],Lw[m]
//	AppendText/N=text0 str
//	ModifyGraph mode=4,marker=19,msize=2
//	return 0
//End
////
//Static Function/T peakStats(yw,xw)
//	Wave yw,xw
//
//	WaveStats/Q yw
//	PulseStats/B=1/F=0.5/L=(0,V_max)/P/Q yw
//	if (V_flag!=1 || numtype(V_PulseLoc1+V_PulseLoc2) || V_PulsePolarity<1)	// must have exactly 2 level crossings
//		return ""
//	endif
//	Variable xlo,xhi, xmax
//	if (WaveExists(xw))
//		xlo = xw[V_PulseLoc1]
//		xhi = xw[V_PulseLoc2]
//		xmax = xw[(V_maxloc-DimOffset(yw,0))/DimDelta(yw,0)]
//	else
//		xlo = DimOffset(xw,0) + V_PulseLoc1*DimDelta(xw,0)
//		xhi = DimOffset(xw,0) + V_PulseLoc2*DimDelta(xw,0)
//		xmax = V_maxloc
//	endif
//	String list = ReplaceNumberByKey("fwhm","",abs(xhi-xlo))
//	list = ReplaceNumberByKey("center",list,(xlo+xhi)/2)
//	list = ReplaceNumberByKey("centerPnt",list,(V_PulseLoc1+V_PulseLoc2)/2)
//	list = ReplaceNumberByKey("max",list,V_max)
//	list = ReplaceNumberByKey("Xmax",list,xmax)
//	return list
//End

